module Date exposing (..)

import String exposing (fromInt)

type Weekday
    = Mon
    | Tue
    | Wed
    | Thu
    | Fri
    | Sat
    | Sun


type alias Day =
    Int


type alias Month =
    Int


type alias Year =
    Int


type alias Date =
    ( Day, Month, Year )


year : Date -> Year
year ( _, _, y ) =
    y


month : Date -> Month
month ( _, m, _ ) =
    m


day : Date -> Day
day ( d, _, _ ) =
    d


lt : Date -> Date -> Bool
lt ( d1, m1, y1 ) ( d2, m2, y2 ) =
    (y1 < y2) || (y1 == y2 && m1 < m2) || (y1 == y2 && m1 == m2 && d1 < d2)


eq : Date -> Date -> Bool
eq ( d1, m1, y1 ) ( d2, m2, y2 ) =
    y1 == y2 && m1 == m2 && d1 == d2


gt : Date -> Date -> Bool
gt ( d1, m1, y1 ) ( d2, m2, y2 ) =
    (y1 > y2) || (y1 == y2 && m1 > m2) || (y1 == y2 && m1 == m2 && d1 > d2)


toString : Date -> String
toString ( d, m, y ) =
    let
        d0 =
            if (d // 10) == 0 then
                "0"

            else
                ""

        m0 =
            if (m // 10) == 0 then
                "0"

            else
                ""
    in
    d0 ++ fromInt d ++ "." ++ m0 ++ fromInt m ++ "." ++ fromInt y


next : Date -> Date
next ( d, m, y ) =
    case ( d, m, y ) of
        ( 31, 12, _ ) ->
            ( 1, 1, y + 1 )

        ( 30, 11, _ ) ->
            ( 1, 12, y )

        ( 31, 10, _ ) ->
            ( 1, 11, y )

        ( 30, 9, _ ) ->
            ( 1, 10, y )

        ( 31, 8, _ ) ->
            ( 1, 9, y )

        ( 31, 7, _ ) ->
            ( 1, 8, y )

        ( 30, 6, _ ) ->
            ( 1, 7, y )

        ( 31, 5, _ ) ->
            ( 1, 6, y )

        ( 30, 4, _ ) ->
            ( 1, 5, y )

        ( 31, 3, _ ) ->
            ( 1, 4, y )

        ( 29, 2, _ ) ->
            ( 1, 3, y )

        ( 28, 2, _ ) ->
            if leapyear y then
                ( 29, 2, y )

            else
                ( 1, 3, y )

        ( 31, 1, _ ) ->
            ( 1, 2, y )

        _ ->
            ( d + 1, m, y )


prev : Date -> Date
prev ( d, m, y ) =
    case ( d, m, y ) of
        ( 1, 12, _ ) ->
            ( 30, 11, y )

        ( 1, 11, _ ) ->
            ( 31, 10, y )

        ( 1, 10, _ ) ->
            ( 30, 9, y )

        ( 1, 9, _ ) ->
            ( 31, 8, y )

        ( 1, 8, _ ) ->
            ( 31, 7, y )

        ( 1, 7, _ ) ->
            ( 30, 6, y )

        ( 1, 6, _ ) ->
            ( 31, 5, y )

        ( 1, 5, _ ) ->
            ( 30, 4, y )

        ( 1, 4, _ ) ->
            ( 31, 3, y )

        ( 1, 3, _ ) ->
            if leapyear y then
                ( 29, 2, y )

            else
                ( 28, 2, y )

        ( 1, 2, _ ) ->
            ( 31, 1, y )

        ( 1, 1, _ ) ->
            ( 31, 12, y - 1 )

        _ ->
            ( d - 1, m, y )


leapyear : Year -> Bool
leapyear y =
    (modBy 4 y == 0) && ((modBy 100 y /= 0) || (modBy 400 y == 0))

{-
BROKEN IMPLEMENTATION:

sub : Date -> Date -> Int
sub d1 d2 =
    subHelp 0 d1 d2



{- um optimierbare End-Rekursion zu erreichen: Akkumulator als Argument -}


subHelp : Int -> Date -> Date -> Int
subHelp diff d1 d2 =
    if gt d1 d2 then
        subHelp (diff + 1) d1 (next d2)

    else if lt d1 d2 then
        subHelp (diff - 1) d1 (prev d2)

    else
        diff

-}


sub : Date -> Date -> Int
sub date1 date2 = convertDateToDayNumber date1 - convertDateToDayNumber date2


-- https://web.archive.org/web/20170507133619/https://alcor.concordia.ca/~gpkatch/gdate-algorithm.html

convertDateToDayNumber : Date -> Int
convertDateToDayNumber (a, b, c) =
    let m = (modBy 12 (b + 9))
        y = c - m // 10
    in 365 * y + y // 4 - y // 100 + y // 400 + (m * 306 + 5) // 10 + ( a - 1 )

convertDayNumberToDate : Int -> Date
convertDayNumberToDate g =
    let y = (10000 * g + 14780) // 3652425
        ddd = g - (365 * y + y // 4 - y // 100 + y // 400)
    in
        if (ddd < 0) then
            let y2 = y - 1 -- y = y2
                ddd2 = g - (365 * y2 + y2 // 4 - y2 // 100 + y2 // 400)  -- ddd = ddd2
                mi = (100 * ddd2 + 52) // 3060
                mm = (modBy (12) (mi + 2)) + 1
                y3 = y2 + (mi + 2) // 12  -- y = y3
                dd = ddd2 - (mi * 306 + 5) // 10 + 1
            in (dd, mm, y3)
        else
            let mi = (100 * ddd + 52) // 3060
                mm = (modBy (12) (mi + 2)) + 1
                y2 = y + (mi + 2) // 12  -- y = y2
                dd = ddd - (mi * 306 + 5) // 10 + 1
            in (dd, mm, y2)

