module Simple exposing (main)

import Browser
import Html exposing (..)
import Html.Events exposing (onClick, onInput)


type Direction
    = LessThan
    | GreaterThan


type Msg
    = Change String
    | Inc
    | Filter Direction


type alias Model =
    { numbers : List Int
    , limit : Maybe Int
    , filter : Int -> Bool
    }


initialmodel : Model
initialmodel =
    { numbers = []
    , filter = \_ -> True
    , limit = Nothing
    }


view : Model -> Html Msg
view model =
    div []
        [ h1 []
            [ text "Zahlen" ]
        , p []
            [ text "Dies ist ein Beispiel." ]
        , button [ onClick Inc ]
            [ text "add" ]
        , button [ onClick (Filter LessThan) ]
            [ text "filter less than" ]
        , input [ onInput Change ]
            []
        , button [ onClick (Filter GreaterThan) ]
            [ text "filter greater than" ]
        , ul []
            (model.numbers
                |> List.filter model.filter
                |> List.map viewNumber
            )
        ]


viewNumber : Int -> Html Msg
viewNumber i =
    li [] [ text (String.fromInt i) ]


functionBuilder : Model -> Direction -> (Int -> Bool)
functionBuilder model dir =
    \i ->
        case model.limit of
            Just n ->
                if dir == LessThan then
                    i > n

                else
                    n > i

            Nothing ->
                True


update : Msg -> Model -> Model
update msg model =
    case msg of
        Change limit ->
            { model | limit = String.toInt limit }

        Filter dir ->
            { model | filter = functionBuilder model dir }

        Inc ->
            { model
                | numbers =
                    case List.maximum model.numbers of
                        Just max ->
                            (max + 1) :: model.numbers

                        Nothing ->
                            0 :: model.numbers
            }


main : Program () Model Msg
main =
    Browser.sandbox { init = initialmodel, update = update, view = view }
